/*
BSL-Scripter-3.1.0.0

==Interpreter.cpp==

Copyright (c) 2008  2015 Texas Instruments Incorporated

All rights reserved not granted herein.

Limited License.

Texas Instruments Incorporated grants a world-wide, royalty-free,
non-exclusive license under copyrights and patents it now or
hereafter owns or controls to make, have made, use, import,
offer to sell and sell ("Utilize") this software subject to
the terms herein.  With respect to the foregoing patent license,
such license is granted  solely to the extent that any such patent
is necessary to Utilize the software alone.  The patent license
shall not apply to any combinations which include this software,
other than combinations with devices manufactured by or for TI
(TI Devices). No hardware patent is licensed hereunder.

Redistributions must preserve existing copyright notices and
reproduce this license (including the above copyright notice
and the disclaimer and (if applicable) source code license
limitations below) in the documentation and/or other materials
provided with the distribution

Redistribution and use in binary form, without modification,
are permitted provided that the following conditions are met:

*	No reverse engineering, decompilation, or disassembly of
this software is permitted with respect to any software
provided in binary form.

*	any redistribution and use are licensed by TI for use only
with TI Devices.

*	Nothing shall obligate TI to provide you with source code
for the software licensed and provided to you in object code.

If software source code is provided to you, modification and
redistribution of the source code are permitted provided that
the following conditions are met:

*	any redistribution and use of the source code, including any
resulting derivative works, are licensed by TI for use only
with TI Devices.

*	any redistribution and use of any object code compiled from
the source code and any resulting derivative works, are licensed
by TI for use only with TI Devices.

Neither the name of Texas Instruments Incorporated nor the names
of its suppliers may be used to endorse or promote products derived
from this software without specific prior written permission.

DISCLAIMER.

THIS SOFTWARE IS PROVIDED BY TI AND TIS LICENSORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TIS LICENSORS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Version:	Release:		Author:
3.0.1.0		08/07/2015		Fatmawati Santosa
3.1.0.0		10/26/2015		Fatmawati Santosa
*/

#include "stdafx.h"

Interpreter::Interpreter()
{
}

Interpreter::~Interpreter()
{

}

void Interpreter::clearParamsInput()
{
	for (uint8_t i = 0; i < PARAMS_SIZE; i++)
	{
		actualCommandParams.paramsInput[i] = "";
	}
}

bool Interpreter::checkCommentLine(std::string command)
{
	uint8_t pos = command.find_first_of("//");
	bool lineEmpty = command.empty();
	return ((pos == 0) || lineEmpty);
}

void Interpreter::checkModeParameters()
{
	actualCommandParams.enLog = false;
	boost::to_lower(actualCommandParams.paramsInput[0]);
	if (actualCommandParams.paramsInput[0] == "log")
	{
		actualCommandParams.enLog = true;
	}
	else if (actualCommandParams.paramsInput[0] != "mode")
	{
		throw std::runtime_error("Please enter the MODE command to start the scripter...");
	}
	else
	{
		actualCommandParams.cryptoMode = false;
		boost::to_lower(actualCommandParams.paramsInput[1]);
		if (actualCommandParams.paramsInput[1] == "crypto")
		{
			//only valid for FRxx
			actualCommandParams.family = Family::FRxx;
			boost::to_lower(actualCommandParams.paramsInput[2]);
			if ((actualCommandParams.paramsInput[2] == familyDefs[6]) || (actualCommandParams.paramsInput[2] == familyDefs[7]))
			{
				actualCommandParams.cryptoMode = true;
				boost::to_lower(actualCommandParams.paramsInput[3]);
				//Baud rate of Crypto Mode is set to default at the start
				if (actualCommandParams.paramsInput[3] == "uart")
				{
					actualCommandParams.protocol = Protocol::UART;
					if (actualCommandParams.paramsInput[4] == "9600")
					{
						actualCommandParams.commPort = actualCommandParams.paramsInput[5];
						if (actualCommandParams.paramsInput[6] != "")
						{
							boost::to_lower(actualCommandParams.paramsInput[6]);
							if (actualCommandParams.paramsInput[6] == "parity")
							{
								actualCommandParams.enParity = true;
							}
						}
					}
					else
					{
						actualCommandParams.commPort = actualCommandParams.paramsInput[4];
						if (actualCommandParams.paramsInput[5] != "")
						{
							boost::to_lower(actualCommandParams.paramsInput[5]);
							if (actualCommandParams.paramsInput[5] == "parity")
							{
								actualCommandParams.enParity = true;
							}
						}
					}

					actualCommandParams.baudRate = 9600;
				}
				else if (actualCommandParams.paramsInput[3] == "i2c")
				{
					actualCommandParams.protocol = Protocol::I2C;
					if (actualCommandParams.paramsInput[4] == "100000")
					{
						actualCommandParams.baudRate = 100000;
						actualCommandParams.commPort = actualCommandParams.paramsInput[5];
						if (actualCommandParams.paramsInput[6] != "")
						{
							boost::to_lower(actualCommandParams.paramsInput[6]);
							if (actualCommandParams.paramsInput[6] == "parity")
							{
								actualCommandParams.enParity = true;
							}
						}

					}
					else if (actualCommandParams.paramsInput[4] == "400000")
					{
						actualCommandParams.baudRate = 400000;
						actualCommandParams.commPort = actualCommandParams.paramsInput[5];
						if (actualCommandParams.paramsInput[6] != "")
						{
							boost::to_lower(actualCommandParams.paramsInput[6]);
							if (actualCommandParams.paramsInput[6] == "parity")
							{
								actualCommandParams.enParity = true;
							}
						}
					}
					else
					{
						actualCommandParams.baudRate = 100000;
						actualCommandParams.commPort = actualCommandParams.paramsInput[4];
						if (actualCommandParams.paramsInput[5] != "")
						{
							boost::to_lower(actualCommandParams.paramsInput[5]);
							if (actualCommandParams.paramsInput[5] == "parity")
							{
								actualCommandParams.enParity = true;
							}
						}
					}
				}
				else
				{
					throw std::runtime_error("Crypto BSL does not supported by any protocols than UART and I2C");
				}
				actualCommandParams.maxBufferSize = 240;
			}
			else
			{
				throw std::runtime_error("Crypto BSL does not supported by any families than FRxx");
			}
		}
		else
		{
			//check the family
			bool familyValid = false;
			boost::to_lower(actualCommandParams.paramsInput[1]);
			for (uint8_t i = 0; i < NUM_FAMILIES_AVAILABLE; i++)
			{
				if (actualCommandParams.paramsInput[1] == familyDefs[i])
				{
					if ((i == 0) || (i == 1))      actualCommandParams.family = Family::F543x;
					else if ((i == 2) || (i == 3)) actualCommandParams.family = Family::F5xx;
					else if ((i == 4) || (i == 5)) actualCommandParams.family = Family::F6xx;
					else if ((i == 6) || (i == 7)) actualCommandParams.family = Family::FRxx;
					else if ((i == 8) || (i == 9)) actualCommandParams.family = Family::P4xx;
					else                           actualCommandParams.family = Family::ROM;
					familyValid = true;
					break;
				}
			}
			if (familyValid == false)
			{
				throw std::runtime_error("Family is not defined...");
			}

			//check protocol
			bool protocolValid = false;
			boost::to_lower(actualCommandParams.paramsInput[2]);
			for (uint8_t i = 0; i < NUM_PROTOCOLS_AVAILABLE; i++)
			{
				if (actualCommandParams.paramsInput[2] == protocolDefs[i])
				{
					if (i == 0)      actualCommandParams.protocol = Protocol::UART;
					else if (i == 1) actualCommandParams.protocol = Protocol::I2C;
					else if (i == 2) actualCommandParams.protocol = Protocol::SPI;
					else             actualCommandParams.protocol = Protocol::USB;
					protocolValid = true;
					break;
				}
			}

			if (protocolValid == false)
			{
				actualCommandParams.protocol = Protocol::UART;
				actualCommandParams.baudRate = 9600;
				actualCommandParams.maxBufferSize = 240;
				actualCommandParams.commPort = actualCommandParams.paramsInput[2];
				if (actualCommandParams.numOfParams == 3)
				{
					boost::to_lower(actualCommandParams.paramsInput[3]);
					if (actualCommandParams.paramsInput[3] == "parity")
					{
						actualCommandParams.enParity = true;
					}
					else
					{
						actualCommandParams.enParity = false;
					}
				}
				return;
			}
			// Check baudrate
			if (actualCommandParams.protocol != Protocol::USB)
			{
				// In P432 family the higher baud rate could be setup at the MODE command
				if ((actualCommandParams.family == Family::P4xx) && (actualCommandParams.paramsInput[4] != ""))
				{
					actualCommandParams.baudRate = (uint32_t)stoi(actualCommandParams.paramsInput[3]);
					bool baudRateValid = checkBaudRate(actualCommandParams.protocol, actualCommandParams.baudRate);
					if (!baudRateValid)
					{
						throw std::runtime_error("Baud Rate value is not valid");
					}
					actualCommandParams.commPort = actualCommandParams.paramsInput[4];
					actualCommandParams.enParity = false;
					if (actualCommandParams.paramsInput[5] == "PARITY")
					{
						actualCommandParams.enParity = true;
					}
				}
				else
				{
					// No baud rate value included in parameters
					if (actualCommandParams.paramsInput[4] == "")
					{
						if (actualCommandParams.protocol == Protocol::UART)	actualCommandParams.baudRate = 9600;
						else if (actualCommandParams.protocol == Protocol::I2C)	actualCommandParams.baudRate = 100000;
						else actualCommandParams.baudRate = 125000;

						actualCommandParams.commPort = actualCommandParams.paramsInput[3];
						actualCommandParams.enParity = false;
						if (actualCommandParams.paramsInput[4] == "PARITY")
						{
							actualCommandParams.enParity = true;
						}

					}
					// Baud rate written in parameters, but for families other than MSP432
					// default value should be set
					else
					{
						if ((actualCommandParams.protocol == Protocol::UART) &&
							(actualCommandParams.paramsInput[3] == "9600"))
						{
							actualCommandParams.baudRate = 9600;
						}
						else if (actualCommandParams.protocol == Protocol::I2C) 
						{
						 	if (actualCommandParams.paramsInput[3] == "100000") actualCommandParams.baudRate = 100000;
							else if (actualCommandParams.paramsInput[3] == "400000") actualCommandParams.baudRate = 400000;
							else  std::runtime_error("Baud rate must be set to default for initialization...");
						}
						else
						{
							throw std::runtime_error("Baud rate must be set to default for initialization...");
						}
						actualCommandParams.commPort = actualCommandParams.paramsInput[4];
						actualCommandParams.enParity = false;
						if (actualCommandParams.paramsInput[5] == "PARITY")
						{
							actualCommandParams.enParity = true;
						}
					}
				}
				actualCommandParams.maxBufferSize = 240;
			}
			else
			{
				actualCommandParams.maxBufferSize = 48;
			}
		}
	}
}

void Interpreter::interpretCommandParams(string command)
{
	Interpreter::clearParamsInput();

	boost::char_separator<char> sep(",\t\r ");
	boost::tokenizer<boost::char_separator<char>> tokens(command, sep);

	actualCommandParams.numOfParams = 0;
	for (const string& t : tokens)
	{
		actualCommandParams.paramsInput[actualCommandParams.numOfParams] = t;
		actualCommandParams.numOfParams++;
	}

	actualCommandParams.repeatMode = 0;
	if (actualCommandParams.paramsInput[0] == "MODE")
	{
		actualCommandParams.repeatMode = 1;
	}
	actualCommandParams.enLog = false;
	if (actualCommandParams.paramsInput[0] == "LOG")
	{
		actualCommandParams.enLog = true;
	}
	actualCommandParams.numOfParams = actualCommandParams.numOfParams - 1;
}


void Interpreter::matchCommand()
{
	bool match = false;

	for (uint16_t i = 0; i < NUM_COMMANDS_AVAILABLE; i++)
	{
		boost::to_lower(actualCommandParams.paramsInput[0]);
		if (boost::algorithm::iequals(actualCommandParams.paramsInput[0], commandDefs[i].commandText))
		{
			uint8_t ui8ValidNumOfParams = commandDefs[i].numOfParamsMand + commandDefs[i].numOfParamsOpt;

			if ((ui8ValidNumOfParams == actualCommandParams.numOfParams) ||
				(commandDefs[i].numOfParamsMand == actualCommandParams.numOfParams))
			{
				actualCommandParams.command = retCommandIdx(i);
				match = true;
				break;
			}
			else
			{
				throw "Parameters are invalid...";
			}
		}
	}

	if (match == false)
	{
		throw std::runtime_error("Command is invalid...");
	}
}
  
ActualCommandParams Interpreter::retActualCommandParams()
{
	return actualCommandParams;
}

void Interpreter::setupNewBaudRate(uint32_t newBaudRate)
{
	actualCommandParams.baudRate = newBaudRate;
}

std::string Interpreter::retResponse(uint8_t *buf, uint8_t *headerBuf)
{
	std::string temp;

	switch (actualCommandParams.command)
	{
	case (Command::CHANGE_BAUD_RATE) :
		temp = changeBaudRate(buf, headerBuf);
		break;

	case (Command::CRC_CHECK_32) :
		temp = crcCheck32(buf, headerBuf);
		break;

	case (Command::CRC_CHECK) :
		temp = crcCheck(buf, headerBuf);
		break;

	case (Command::ERASE_SEGMENT_32) :
		temp = eraseSegment32(buf, headerBuf);
		break;

	case (Command::ERASE_SEGMENT) :
		temp = eraseSegment(buf, headerBuf);
		break;

	case (Command::SET_PC) :
		temp = setPC(buf, headerBuf);
		break;

	case (Command::SET_PC_32) :
		temp = setPC(buf, headerBuf);
		break;

	case (Command::MASS_ERASE) :
		temp = massErase(buf, headerBuf);
		break;

	case (Command::REBOOT_RESET) :
		temp = rebootReset(buf, headerBuf);
		break;

	case (Command::RX_DATA_BLOCK_FAST) :
		temp = rxDataBlockFast(buf, headerBuf);
		break;

	case (Command::RX_DATA_BLOCK) :
		temp = rxDataBlock(buf, headerBuf);
		break;

	case (Command::RX_DATA_BLOCK_32) :
		temp = rxDataBlock32(buf, headerBuf);
		break;

	case (Command::RX_PASSWORD) :
		temp = rxPassword(buf, headerBuf);
		break;

	case (Command::RX_PASSWORD_32) :
		temp = rxPassword32(buf, headerBuf);
		break;

	case (Command::TOGGLE_INFO) :
		temp = toggleInfo(buf, headerBuf);
		break;


	case (Command::TX_DATA_BLOCK_32) :
		temp = txDataBlock32(buf, headerBuf);
		break;

	case (Command::TX_DATA_BLOCK) :
		temp = txDataBlock(buf, headerBuf);
		break;

	case (Command::TX_BSL_VERSION) :
		temp = txBslVersion(buf, headerBuf);
		break;

	case (Command::TX_BSL_VERSION_32) :
		temp = txBslVersion32(buf, headerBuf);
		break;

	case (Command::TX_BUFFER_SIZE) :
		temp = txBufferSize(buf, headerBuf);
		break;

	case (Command::RX_SECURE_DATA_BLOCK) :
		temp = rxDataBlock(buf, headerBuf);
		break;

	default:
		temp = unknownCommand();
		break;
	}

	return temp;
}


bool Interpreter::isRxDataPassInputCommand()
{
	if (((actualCommandParams.command == Command::RX_DATA_BLOCK) ||
		(actualCommandParams.command == Command::RX_DATA_BLOCK_32) ||
		(actualCommandParams.command == Command::RX_PASSWORD) ||
		(actualCommandParams.command == Command::RX_PASSWORD_32) ||
		(actualCommandParams.command == Command::RX_DATA_BLOCK_FAST) ||
		(actualCommandParams.command == Command::RX_SECURE_DATA_BLOCK)) &&
		(actualCommandParams.paramsInput[1] != ""))
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool Interpreter::isRxPassword()
{
	if ((actualCommandParams.command == Command::RX_PASSWORD) ||
		(actualCommandParams.command == Command::RX_PASSWORD_32))
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool Interpreter::isRxPassDefaultCommand()
{
	if (((actualCommandParams.command == Command::RX_PASSWORD) ||
		(actualCommandParams.command == Command::RX_PASSWORD_32)) &&
		(actualCommandParams.paramsInput[1] == ""))
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool Interpreter::isTxDataCommand()
{
	if ((actualCommandParams.command == Command::TX_DATA_BLOCK) ||
		(actualCommandParams.command == Command::TX_DATA_BLOCK_32))
	{
		return true;
	}
	else
	{
		return false;
	}
}

bool Interpreter::isCommandReqNoResponse()
{
	if ((actualCommandParams.command == Command::DELAY) ||
		(actualCommandParams.command == Command::VERBOSE))
	{
		return true;
	}
	else
	{
		return false;
	}
}

/*******************************************************************************
*Class:       Interpreter
*Function:    retCrcBsl
*Description: return the crc value from the response
*Parameters:  none
*Returns:     uint16_t crcBsl: crc from BSL device
*******************************************************************************/
uint16_t Interpreter::retCrcBsl()
{
	return crcBsl;
}

Command Interpreter::retCommandIdx(uint16_t idx)
{
	switch (idx)
	{
	case(0) : return (Command::CHANGE_BAUD_RATE); break;
	case(1) : return (Command::CRC_CHECK_32); break;
	case(2) : return (Command::CRC_CHECK); break;
	case(3) : return (Command::DELAY); break;
	case(4) : return (Command::ERASE_CHECK); break;
	case(5) : return (Command::ERASE_MAIN_OR_INFO); break;
	case(6) : return (Command::ERASE_SEGMENT_32); break;
	case(7) : return (Command::ERASE_SEGMENT); break;
	case(8) : return (Command::SET_PC_32); break;
	case(9) : return (Command::SET_PC); break;
	case(10) : return (Command::MASS_ERASE); break;
	case(11) : return (Command::REBOOT_RESET); break;
	case(12) : return (Command::RX_DATA_BLOCK_FAST); break;
	case(13) : return (Command::RX_DATA_BLOCK_32); break;
	case(14) : return (Command::RX_DATA_BLOCK); break;
	case(15) : return (Command::RX_PASSWORD_32); break;
	case(16) : return (Command::RX_PASSWORD); break;
	case(17) : return (Command::SET_MEMORY_OFFSET); break;
	case(18) : return (Command::TOGGLE_INFO); break;
	case(19) : return (Command::TX_DATA_BLOCK_32); break;
	case(20) : return (Command::TX_DATA_BLOCK); break;
	case(21) : return (Command::TX_BSL_VERSION_32); break;
	case(22) : return (Command::TX_BSL_VERSION); break;
	case(23) : return (Command::TX_BUFFER_SIZE); break;
	case(24) : return (Command::VERBOSE); break;
	case(25) : return (Command::RX_SECURE_DATA_BLOCK); break;
	default: return (Command::UNKNOWN_COMMAND); break;
	}
}


bool Interpreter::checkBaudRate(Protocol protocol, uint32_t baudRate)
{
	bool valid = false;
	if (protocol == Protocol::UART)
	{
		for (uint8_t i = 0; i < NUM_BR_UART_AVAILABLE; i++)
		{
			if (baudRate == baudRateUart[i])
			{
				valid = true;
				break;
			}
		}
	}
	else if (protocol == Protocol::I2C)
	{
		for (uint8_t i = 0; i < NUM_BR_I2C_AVAILABLE; i++)
		{
			if (baudRate == baudRateI2c[i])
			{
				valid = true;
				break;
			}
		}
	}
	else
	{
		for (uint8_t i = 0; i < NUM_BR_SPI_AVAILABLE; i++)
		{
			if (baudRate == baudRateSpi[i])
			{
				valid = true;
				break;
			}
		}
	}
	return valid;
}

std::string Interpreter::checkHeader(uint8_t *bufHeader, uint8_t *messageReply, uint8_t *coreResponse)
{
	std::string retString;
	switch (*bufHeader)
	{
		case (0x00) :
			{
				if (*messageReply == 0x3B)
				{
					if (*coreResponse == FLASH_WRITE_CHECK_FAILED) retString = "Flash write check failed.";
					else if (*coreResponse == FLASH_FAIL_BIT_SET) retString = "Flash fail bit set.";
					else if (*coreResponse == VOLTAGE_CHANGE_DURING_PROGRAM) retString = "Voltage change during program.";
					else if (*coreResponse == BSL_LOCKED) retString = "BSL locked.";
					else if (*coreResponse == BSL_PASSWORD_ERROR) retString = "Password is incorrect.";
					else if (*coreResponse == BYTE_WRITE_FORBIDDEN) retString = "BSL byte write forbidden.";
					else retString = "Unknown command.";
				}
				else
				{
					retString = "";
				}
			}
			break;

		case (0x51) :
			retString = "Error Header incorrect.";
			break;
 		case (0x52) :
			retString = "Error Checksum incorrect.";
			break;
 		case (0x53) :
			retString = "Error Packet Size is zero.";
			break;
		case (0x54) :
			retString = "Error Packet Size exceeds the buffer size.";
			break;
 		case (0x55) :
			retString = "Unknown error.";
			break;
 		case (0x56) :
			retString = "Unknown baud rate.";
			break;
 		default:
			retString = " ";
			break;
	}
	return retString;
}

std::string Interpreter::intToHex(uint8_t input)
{
	std::stringstream stream;
	stream << std::setfill('0')
		<< std::setw(sizeof(input) * 2)
		<< std::hex
		<< (uint32_t)input;
	return stream.str();
}

std::string Interpreter::intToHexCrc(uint16_t input)
{
	std::stringstream stream;
	stream << std::setfill('0')
		<< std::hex
		<< (uint32_t)input;
	return stream.str();
}


std::string Interpreter::intToString(uint8_t input)
{
	std::stringstream stream;
	stream << std::setfill('0')
		<< std::setw(sizeof(input) * 2)
		<< std::hex << "0x"
		<< (uint32_t)input;
	return stream.str();
}

std::string Interpreter::changeBaudRate(uint8_t *buf, uint8_t *ackByte)
{
 	std::string retString;
	if (buf[0] == 0x00)
	{
		retString= "BSL ACK is received.";
	}
	else
	{
		retString = "Change baud rate error : " + intToString(buf[0]);
	}
	return retString;
}
 
std::string Interpreter::crcCheck32(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString;
	std::string temp;
	if (buf[0] == BSL_DATA_REPLY)
	{
		crcBsl = buf[1] | (buf[2] << 8);
		temp = "0x" + intToHexCrc(crcBsl);
		retString = "CRC check result is : " + temp;

	}
	else
	{
		retString = checkHeader(&ackByte[0], &buf[0], &buf[1]);
		return retString;
	}

	boost::to_upper(actualCommandParams.paramsInput[3]);
	boost::to_upper(temp);

	if ((actualCommandParams.paramsInput[3] != "") &&
		(temp == actualCommandParams.paramsInput[3]))
	{
		retString += "  CRC is matched";
	}
	else if ((actualCommandParams.paramsInput[3] != "") &&
		(temp != actualCommandParams.paramsInput[3]))
	{
		retString += "  CRC is not matched";
	}
	else
	{
		retString += "  ";
	}
	return retString;
}

std::string Interpreter::crcCheck(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString;
	std::string temp;
	if 	(buf[0] == BSL_DATA_REPLY)
	{
		crcBsl = buf[1] | (buf[2] << 8);
		temp = "0x" + intToHexCrc(crcBsl);
		retString = "CRC check result is : " + temp;

	}
	else
	{
		retString = checkHeader(&ackByte[0], &buf[0], &buf[1]);

		return retString;
  	}

	boost::to_upper(actualCommandParams.paramsInput[3]);
	boost::to_upper(temp);

	if ((actualCommandParams.paramsInput[3] != "") &&
		(temp == actualCommandParams.paramsInput[3]))
	{
		retString += "  CRC is matched";
	}
	else if ((actualCommandParams.paramsInput[3] != "") &&
		     (temp != actualCommandParams.paramsInput[3]))
	{
		retString += "  CRC is not matched";
	}
	else
	{
		retString += "  ";
	}
	return retString;
}


std::string Interpreter::eraseSegment32(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString;

	if ((buf[0] == BSL_MESSAGE_REPLY) &&
		(buf[1] == 0x00)
		)
	{
		retString = "Erase segment done.";
	}
	else
	{
		retString = checkHeader(&ackByte[0], &buf[0], &buf[1]);
	}
	return retString;
}


std::string Interpreter::eraseSegment(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString;

	if ((buf[0] == BSL_MESSAGE_REPLY) &&
		(buf[1] == 0x00)
		)
	{
		retString = "Erase segment done.";
	}
	else
	{
		retString = checkHeader(&ackByte[0], &buf[0], &buf[1]);
	}
	return retString;
}
std::string Interpreter::setPC(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString = "Set PC is sent.";
	return retString;
}

std::string Interpreter::massErase(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString = "";

	if (actualCommandParams.family == Family::FRxx) 
	{
		//resetComm = true;
		retString = "Mass erase done.";
	}
	else if ((buf[0] == BSL_MESSAGE_REPLY) &&
		(buf[1] == 0x00))
	{
		retString = "Mass erase done.";
	}
	else
	{
		retString = checkHeader(&ackByte[0], &buf[0], &buf[1]);
	}


	return retString;
}

std::string Interpreter::rxDataBlockFast(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString;
	if (actualCommandParams.protocol == Protocol::USB)
	{
		retString = "BSL ACK is received.";
	}
	else
	{
		retString = checkHeader(&ackByte[0], &buf[0], &buf[1]);
	}
	return retString;
}

std::string Interpreter::rxDataBlock(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString;

	if ((buf[0] == BSL_MESSAGE_REPLY) && (buf[1] == 0x00))
	{
		retString = "Data received by BSL.";
	}
	else
	{
		retString = checkHeader(&ackByte[0], &buf[0], &buf[1]);
	}
	return retString;
}

std::string Interpreter::rxDataBlock32(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString;

	if ((buf[0] == BSL_MESSAGE_REPLY) && (buf[1] == 0x00))
	{
		retString = "Data received by BSL.";
	}
	else
	{
		retString = checkHeader(&ackByte[0], &buf[0], &buf[1]);
	}
	return retString;
}

std::string Interpreter::rxPassword32(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString;

	if ((buf[0] == BSL_MESSAGE_REPLY) &&
		(buf[1] == 0x00))

	{
		retString = "Password is correct.";
	}
	else
	{
		retString = checkHeader(&ackByte[0], &buf[0], &buf[1]);
	}
	return retString;
}

std::string Interpreter::rxPassword(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString;

	if ((buf[0] == BSL_MESSAGE_REPLY) &&
		(buf[1] == 0x00))

	{
		retString = "Password is correct.";
	}
	else
	{
		retString = checkHeader(&ackByte[0], &buf[0], &buf[1]);
	}
	return retString;
}

std::string Interpreter::toggleInfo(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString;
	if ((buf[0] == BSL_MESSAGE_REPLY) && (buf[1] == 0x00))

	{
		retString = "Toggle info succeed.";
	}
	else
	{
		retString = checkHeader(&ackByte[0], &buf[0], &buf[1]);
	}
	return retString;
}

std::string Interpreter::rebootReset(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString = "";
	return retString;
}

std::string Interpreter::txDataBlock32(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString = "";
	return retString;
}	  

std::string Interpreter::txDataBlock(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString = "";
	return retString;
}

std::string Interpreter::txBslVersion32(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString;
	if (buf[0] == BSL_DATA_REPLY)
	{
		bslVersion32[0] = (uint16_t)buf[1] | (((uint16_t)buf[2]) << 8);
		bslVersion32[1] = (uint16_t)buf[3] | (((uint16_t)buf[4]) << 8);
		bslVersion32[2] = (uint16_t)buf[5] | (((uint16_t)buf[6]) << 8);
		bslVersion32[3] = (uint16_t)buf[7] | (((uint16_t)buf[8]) << 8);
		bslVersion32[4] = (uint16_t)buf[9] | (((uint16_t)buf[10]) << 8);

		retString = "BSL Version is : BSL_Vendor[" + intToHexCrc(bslVersion32[0]) + "],CI[" +
			intToHexCrc(bslVersion32[1]) + "],API[" +
			intToHexCrc(bslVersion32[2]) + "],PI[" +
			intToHexCrc(bslVersion32[3]) + "],Build_ID[" +
			intToHexCrc(bslVersion32[4]) + "]";
	}
	else
	{
		retString = checkHeader(&ackByte[0], &buf[0], &buf[1]);
	}
	return retString;
}

std::string Interpreter::txBslVersion(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString;
	if (buf[0] == BSL_DATA_REPLY)
	{
		bslVersion[0] = buf[1];
		bslVersion[1] = buf[2];
		bslVersion[2] = buf[3];
		bslVersion[3] = buf[4];

		retString = "BSL Version is : BSL_Vendor[" + intToHex(bslVersion[0]) + "],CI[" +
			intToHex(bslVersion[1]) + "],API[" +
			intToHex(bslVersion[2]) + "],PI[" +
			intToHex(bslVersion[3]) + "]";
	}
	else
	{
		retString = checkHeader(&ackByte[0], &buf[0], &buf[1]);
  	}
	return retString;
}

std::string Interpreter::txBufferSize(uint8_t *buf, uint8_t *ackByte)
{
	std::string retString;

	
	if (buf[0] == BSL_DATA_REPLY)
	{
		bufferSize = ((uint16_t)buf[1]) | (((uint16_t)buf[2]) << 8);

		std::string txBufSize = intToHexCrc(bufferSize);

		retString = "Transmit buffer size is : " + txBufSize + " bytes";
	}
	else
	{
		retString = checkHeader(&ackByte[0], &buf[0], &buf[1]);
 	}
	return retString;
}

std::string Interpreter::unknownCommand()
{
	std::string retString = "Unknown command.";
	return retString;
}
