#pragma once

#include "stdafx.h"

class RomTarget : public TargetInterface
{
	/*******************************************************************************
	*Class:       RomTarget
	*Function:    readParameters
	*Description: setup the parameters for each BSL command in RomTarget family
	*Parameters:  ActualCommandParams actualCommandParams: command's parameters
	*             from the interpreter
	*Returns:     none
	*******************************************************************************/
	void readParameters(ActualCommandParams actualCommandParams) override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    changeBaudRate
	*Description: Execution of CHANGE_BAUD_RATE in RomTarget family
	*Parameters:  none
	*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
	*******************************************************************************/
	DataBuffer changeBaudRate() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    crcCheck32
	*Description: Execution of CRC_CHECK_32 in RomTarget family
	*Parameters:  none
	*Returns:     exception
	*******************************************************************************/
	DataBuffer crcCheck32() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    crcCheck
	*Description: Execution of CRC_CHECK in RomTarget family
	*Parameters:  none
	*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
	*******************************************************************************/
	DataBuffer crcCheck() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    delay
	*Description: Execution of DELAY in RomTarget family
	*Parameters:  none
	*Returns:     none
	*******************************************************************************/
	void       delay() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    eraseCheck
	*Description: Execution of ERASE_CHECK in RomTarget family
	*Parameters:  none
	*Returns:     exception
	*******************************************************************************/
	DataBuffer eraseCheck() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    eraseMainOrInfo
	*Description: Execution of ERASE_MAIN_OR_INFO in RomTarget family
	*Parameters:  none
	*Returns:     exception
	*******************************************************************************/
	DataBuffer eraseMainOrInfo() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    eraseSegment32
	*Description: Execution of ERASE_SEGMENT_32 in RomTarget family
	*Parameters:  none
	*Returns:     exception
	*******************************************************************************/
	DataBuffer eraseSegment32() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    eraseSegment
	*Description: Execution of ERASE_SEGMENT in RomTarget family
	*Parameters:  none
	*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
	*******************************************************************************/
	DataBuffer eraseSegment() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    setPc32
	*Description: Execution of SET_PC_32 in RomTarget family
	*Parameters:  none
	*Returns:     exception
	*******************************************************************************/
	DataBuffer setPc32() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    setPc
	*Description: Execution of SET_PC in RomTarget family
	*Parameters:  none
	*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
	*******************************************************************************/
	DataBuffer setPc() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    massErase
	*Description: Execution of MASS_ERASE in RomTarget family
	*Parameters:  none
	*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
	*******************************************************************************/
	DataBuffer massErase() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    rebootReset
	*Description: Execution of REBOOT_RESET in RomTarget family
	*Parameters:  none
	*Returns:     exception
	*******************************************************************************/
	DataBuffer rebootReset() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    rxDataBlockFast
	*Description: Execution of RX_DATA_BLOCK_FAST in RomTarget family
	*Parameters:  none
	*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
	*******************************************************************************/
	DataBuffer rxDataBlockFast() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    rxDataBlock32
	*Description: Execution of RX_DATA_BLOCK_32 in RomTarget family
	*Parameters:  none
	*Returns:     exception
	*******************************************************************************/
	DataBuffer rxDataBlock32() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    rxDataBlock
	*Description: Execution of RX_DATA_BLOCK in RomTarget family
	*Parameters:  none
	*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
	*******************************************************************************/
	DataBuffer rxDataBlock() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    rxPassword32
	*Description: Execution of RX_PASSWORD_32 in RomTarget family
	*Parameters:  none
	*Returns:     exception
	*******************************************************************************/
	DataBuffer rxPassword32() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    rxPassword
	*Description: Execution of RX_PASSWORD in RomTarget family
	*Parameters:  none
	*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
	*******************************************************************************/
	DataBuffer rxPassword() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    setMemoryOffset
	*Description: Execution of SET_MEM_OFFSET in RomTarget family
	*Parameters:  none
	*Returns:     exception
	*******************************************************************************/
	DataBuffer setMemoryOffset() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    toggleInfo
	*Description: Execution of TOGGLE_INFO in RomTarget family
	*Parameters:  none
	*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
	*******************************************************************************/
	DataBuffer toggleInfo() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    txDataBlock32
	*Description: Execution of TX_DATA_BLOCK_32 in RomTarget family
	*Parameters:  none
	*Returns:     exception
	*******************************************************************************/
	DataBuffer txDataBlock32(uint16_t size) override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    txDataBlock
	*Description: Execution of TX_DATA_BLOCK in RomTarget family
	*Parameters:  none
	*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
	*******************************************************************************/
	DataBuffer txDataBlock(uint16_t size) override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    bslVersion32
	*Description: Execution of TX_BSL_VERSION_32 in RomTarget family
	*Parameters:  none
	*Returns:     exception
	*******************************************************************************/
	DataBuffer bslVersion32() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    bslVersion
	*Description: Execution of TX_BSL_VERSION in RomTarget family
	*Parameters:  none
	*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
	*******************************************************************************/
	DataBuffer bslVersion() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    txBufferSize
	*Description: Execution of TX_BUFFER_SIZE in RomTarget family
	*Parameters:  none
	*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
	*******************************************************************************/
	DataBuffer txBufferSize() override;

	/*******************************************************************************
	*Class:		  RomTarget
	*Function:	  rxSecureDataBlock
	*Description: Execution of RX_SECURE_DATA_BLOCK in RomTarget family
	*Parameters:  none
	*Returns:     ataBuffer commandBuffer: BSL command to be transmitted
	*******************************************************************************/
	DataBuffer rxSecureDataBlock() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    initVerbose
	*Description: set the verbose mode to off as initialization
	*Parameters:  none
	*Returns:     none
	*******************************************************************************/
	void       initVerbose() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    verbose
	*Description: toggle verbose mode
	*Parameters:  none
	*Returns:     none
	*******************************************************************************/
	void       verbose() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    retVerbose
	*Description: return verbose mode
	*Parameters:  none
	*Returns:     bool true : verbose on
	*             bool false: verbose off
	*******************************************************************************/
	bool       retVerbose() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    retNewBaudRate
	*Description: return baud rate value
	*Parameters:  none
	*Returns:     uint32_t newBaudRate : new baud rate value
	*******************************************************************************/
	uint32_t   retNewBaudRate() override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:    retNumResponseBytes
	*Description: return number of response byte of respective command
	*Parameters:  none
	*Returns:     uint32_t numResponseBytes :number of response byte
	*******************************************************************************/
	int32_t    retNumResponseBytes() override;


	/*******************************************************************************
	*Class:       RomTarget
	*Function:    setDataBlock
	*Description: copy DataBlock from streaming input to be applied in BSL command
	*Parameters:  DataBlock dataBlockStream: input DataBlock from streaming input
	*Returns:     none
	*******************************************************************************/
	void       setDataBlock(DataBlock dataBlockStream) override;

	/*******************************************************************************
	*Class:       RomTarget
	*Function:	  retTxDataBlockSize
	*Description: return value size of dataBlock
	*Parameters:  none
	*Returns:	  uint32_t : size of dataBlock
	*******************************************************************************/
	uint32_t   retTxDataBlockSize() override;

private:

};