#pragma once

#include "stdafx.h"

class TargetInterface
{
public:
	/***********************************************************************************
	*Class:		  TargetInterface
	*Function:	  callFunc
	*Description: call respective function based on the actualCommandParams from 
	*             interpreter
	*Parameters:  ActualCommandParams actualCommandParams
	*			  uint16_t size	: number of response bytes for specific function
	*Returns:	  none
	***********************************************************************************/
	void callFunc(ActualCommandParams actualCommandParams, uint16_t size);

	/***********************************************************************************
	*Class:		  TargetInterface
	*Function:	  retDataBuffer
	*Description: return dataBuffer that already set by BSL command inside
	*Parameters:  none
	*Returns:	  DataBuffer : DataBuffer to be transmitted to the target device
	***********************************************************************************/
	DataBuffer retDataBuffer();

	/***********************************************************************************
	* Virtual functions that used by derivatives class:
	* P432Target, FlashTarget, FramTarget, and RomTarget
	***********************************************************************************/
	virtual void readParameters(ActualCommandParams actualCommandParams) = 0;
	virtual DataBuffer changeBaudRate() = 0;
	virtual DataBuffer crcCheck32() = 0;
	virtual DataBuffer crcCheck() = 0;
	virtual void delay() = 0;
	virtual DataBuffer eraseCheck() = 0;
	virtual DataBuffer eraseMainOrInfo() = 0;
	virtual DataBuffer eraseSegment32() = 0;
	virtual DataBuffer eraseSegment() = 0;
	virtual DataBuffer setPc32() = 0;
	virtual DataBuffer setPc() = 0;
	virtual DataBuffer massErase() = 0;
	virtual DataBuffer rebootReset() = 0;
	virtual DataBuffer rxDataBlockFast() = 0;
	virtual DataBuffer rxDataBlock32() = 0;
	virtual DataBuffer rxDataBlock() = 0;
	virtual DataBuffer rxPassword32() = 0;
	virtual DataBuffer rxPassword() = 0;
	virtual DataBuffer setMemoryOffset() = 0;
	virtual DataBuffer toggleInfo() = 0;
	virtual DataBuffer txDataBlock32(uint16_t size) = 0;
	virtual DataBuffer txDataBlock(uint16_t size) = 0;
	virtual DataBuffer bslVersion32() = 0;
	virtual DataBuffer bslVersion() = 0;
	virtual DataBuffer txBufferSize() = 0;
	virtual DataBuffer rxSecureDataBlock() = 0;
	virtual void       initVerbose() = 0;
	virtual void       verbose() = 0;
	virtual bool       retVerbose() = 0;
	virtual uint32_t   retNewBaudRate() = 0;
	virtual int32_t    retNumResponseBytes() = 0;
	virtual void       setDataBlock(DataBlock dataBlockStream) = 0;
	virtual uint32_t   retTxDataBlockSize() = 0;


protected:

	// Header bytes command for RomTarget
	const uint8_t ROM_ACK = 0x90;
	const uint8_t ROM_RX_PASSWORD = 0x10;
	const uint8_t ROM_RX_DATA_BLOCK = 0x12;
	const uint8_t ROM_TX_DATA_BLOCK = 0x14;
	const uint8_t ROM_ERASE_SEGMENT = 0x16;
	const uint8_t ROM_ERASE_MAIN_OR_INFO = 0x16;
	const uint8_t ROM_MASS_ERASE = 0x18;
	const uint8_t ROM_ERASE_CHECK = 0x1C;
	const uint8_t ROM_SET_MEM_OFFSET = 0x21;
	const uint8_t ROM_DUMMY_DATA = 0x00;

	// Header bytes command for FlashTarget FramTarget
	const uint8_t RX_DATA_BLOCK = 0x10;
	const uint8_t RX_PASSWORD = 0x11;
	const uint8_t ERASE_SEGMENT = 0x12;
	const uint8_t TOGGLE_INFO = 0x13;
	const uint8_t ERASE_BLOCK = 0x14;
	const uint8_t MASS_ERASE = 0x15;
	const uint8_t CRC_CHECK = 0x16;
	const uint8_t SET_PC = 0x17;
	const uint8_t TX_DATA_BLOCK = 0x18;
	const uint8_t TX_BSL_VERSION = 0x19;
	const uint8_t TX_BUFFER_SIZE = 0x1A;
	const uint8_t RX_DATA_BLOCK_FAST = 0x1B;
	
	
	// Header bytes command for P432Target
	const uint8_t RX_DATA_BLOCK_32 = 0x20;
	const uint8_t RX_PASSWORD_32 = 0x21;
	const uint8_t ERASE_SEGMENT_32 = 0x22;
	const uint8_t REBOOT_RESET = 0x25;
	const uint8_t CRC_CHECK_32 = 0x26;
	const uint8_t SET_PC_32 = 0x27;
	const uint8_t TX_DATA_BLOCK_32 = 0x28;
	const uint8_t TX_BSL_VERSION_32 = 0x19;

	//
	const uint8_t RX_DATA_ENC_KEY = 0x31;
	const uint8_t RX_DATA_BLOCK_ENCRYPTED = 0x30;
	const uint32_t ENC_KEY_ADDRESS = 0x5000;
	const uint32_t ENC_DATA_ADDRESS = 0xA000;

	// Baud rate bytes
	const uint8_t CHANGE_BAUD_RATE = 0x52;
	const uint8_t BAUD430_4800 = 0x01;
	const uint8_t BAUD430_9600 = 0x02;
	const uint8_t BAUD430_19200 = 0x03;
	const uint8_t BAUD430_38400 = 0x04;
	const uint8_t BAUD430_57600 = 0x05;
	const uint8_t BAUD430_115200 = 0x06;

	// Variables

	bool cryptoMode;
	bool verboseToggle;

	uint32_t address;
	uint32_t address32;
	uint32_t length;
	uint32_t newBaudRate;
	uint8_t  commandBaudRate;
	uint32_t delayMs;

	DataBlock dataBlock;
	int32_t numResponseBytes;
	uint32_t totalBytesReceived;
	uint32_t offsetBytesRead;

	DataBuffer dataBuffer;



private:

};

