#pragma once

#include "stdafx.h"

#if defined(_WIN32) || defined(_WIN64)
	# define MSPBSL_ON_WIN
	# define BOOST_ASIO_OPTION_STORAGE DCB
#else
	# define MSPBSL_ON_LINUX
	# define BOOST_ASIO_OPTION_STORAGE termios
#endif

class TESTControl
{
private:

	uint16_t state;

public:
	TESTControl(uint16_t initState)
	{
		state = initState;
	}; // constructor

	boost::system::error_code store(BOOST_ASIO_OPTION_STORAGE& storage, boost::system::error_code& error) const
	{

#if defined(_WIN32) || defined(_WIN64)
		if (state)
		{
			storage.fRtsControl = RTS_CONTROL_DISABLE;
		}
		else
		{
			storage.fRtsControl = RTS_CONTROL_ENABLE;
		}
#elif defined ( __APPLE_ )

		//not yet implemented

#else
		// linux currently untested
		//uint16_t flags;
		//ioctl(fd, TIOCMGET, &flags);
		if (state)
		{
			//flags &= ~TIOCM_RTS;
			storage.c_cflag &= ~CRTSCTS;
		}
		else
		{
			//flags |= TIOCM_RTS;
			storage.c_cflag |= CRTSCTS;
		}
		//tcsetattr( fd, TCSANOW, storage );
		//ioctl(fd, TIOCMSET, &flags);
#endif
		return error;
	}; // store

	boost::system::error_code load(BOOST_ASIO_OPTION_STORAGE& storage, boost::system::error_code& error)
	{

#if defined(_WIN32) || defined(_WIN64)

		if (storage.fRtsControl == RTS_CONTROL_ENABLE)
		{
			state = 0;
		}
		else
		{
			state = 1;
		}

#elif defined ( __APPLE__ )
		
		//not implemented yet

#else
		//uint16_t flags;
		//ioctl(fd, TIOCMGET, &flags);
		//if( flags & TIOCM_RTS )
		if (storage.c_cflag & CRTSCTS)
		{
			state = 0;
		}
		else
		{
			state = 1;
	}
#endif
		return error;
	};
};

class RESETControl
{
private:

	uint16_t state;

public:
	RESETControl(uint16_t initState)
	{
		state = initState;
	};  // constructor
	boost::system::error_code store(BOOST_ASIO_OPTION_STORAGE& storage, boost::system::error_code& error) const
	{
#if defined( MSPBSL_ON_WIN )
		if (state)
		{
			storage.fDtrControl = DTR_CONTROL_ENABLE;
		}
		else
		{
			storage.fDtrControl = DTR_CONTROL_DISABLE;
		}
#elif defined ( MSPBSL_ON_LINUX )
		// linux currently untested
		//uint16_t flags;
		//ioctl(fd, TIOCMGET, &flags);
		if (state)
		{
			//flags  |= TIOCM_DTR;
			storage.c_cflag &= ~CRTSCTS;
		}
		else
		{
			//flags &= ~TIOCM_DTR;
			storage.c_cflag |= CRTSCTS;
		}
		//tcsetattr( fd, TCSANOW, storage );
		//ioctl(fd, TIOCMSET, &flags);

#endif
		return error;
	}; // store

	boost::system::error_code load(BOOST_ASIO_OPTION_STORAGE& storage, boost::system::error_code& error)
	{
#if defined( MSPBSL_ON_WIN )
		if (storage.fDtrControl == DTR_CONTROL_ENABLE)
		{
			state = 1;
		}
		else
		{
			state = 0;
		}

#elif defined ( MSPBSL_ON_LINUX )
		//uint16_t flags;
		//ioctl(fd, TIOCMGET, &flags);
		//if( flags & TIOCM_DTR )
		if (storage.c_cflag & CRTSCTS)
		{
			state = 1;
		}
		else
		{
			state = 0;
		}
#endif
		return error;
	}; // load

}; // class TestControl
