/*
BSL-Scripter-3.1.0.0

==UartComm.cpp==

Copyright (c) 2008  2015 Texas Instruments Incorporated

All rights reserved not granted herein.

Limited License.

Texas Instruments Incorporated grants a world-wide, royalty-free,
non-exclusive license under copyrights and patents it now or
hereafter owns or controls to make, have made, use, import,
offer to sell and sell ("Utilize") this software subject to
the terms herein.  With respect to the foregoing patent license,
such license is granted  solely to the extent that any such patent
is necessary to Utilize the software alone.  The patent license
shall not apply to any combinations which include this software,
other than combinations with devices manufactured by or for TI
(TI Devices). No hardware patent is licensed hereunder.

Redistributions must preserve existing copyright notices and
reproduce this license (including the above copyright notice
and the disclaimer and (if applicable) source code license
limitations below) in the documentation and/or other materials
provided with the distribution

Redistribution and use in binary form, without modification,
are permitted provided that the following conditions are met:

*	No reverse engineering, decompilation, or disassembly of
this software is permitted with respect to any software
provided in binary form.

*	any redistribution and use are licensed by TI for use only
with TI Devices.

*	Nothing shall obligate TI to provide you with source code
for the software licensed and provided to you in object code.

If software source code is provided to you, modification and
redistribution of the source code are permitted provided that
the following conditions are met:

*	any redistribution and use of the source code, including any
resulting derivative works, are licensed by TI for use only
with TI Devices.

*	any redistribution and use of any object code compiled from
the source code and any resulting derivative works, are licensed
by TI for use only with TI Devices.

Neither the name of Texas Instruments Incorporated nor the names
of its suppliers may be used to endorse or promote products derived
from this software without specific prior written permission.

DISCLAIMER.

THIS SOFTWARE IS PROVIDED BY TI AND TIS LICENSORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TIS LICENSORS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Version:	Release:		Author:
3.0.1.0		08/07/2015		Fatmawati Santosa
3.1.0.0		10/26/2015		Fatmawati Santosa

*/
#include "stdafx.h"


#if defined(_WIN32) || defined(_WIN64)

#else
#include <unistd.h>     // UNIX standard function definitions
#include <fcntl.h>      // File control definitions
#include <termios.h>    // POSIX terminal control definitions
#include <sys/ioctl.h>

struct termios2 {
	tcflag_t c_iflag;
	tcflag_t c_oflag;
	tcflag_t c_cflag;
	tcflag_t c_lflag;
	cc_t c_line;
	cc_t c_cc[19];
	speed_t c_ispeed;
	speed_t c_ospeed;
};
#endif

UartComm::UartComm()
	:port(io_service),
	timer(io_service)

{
	resetIdxRead();
}

UartComm::~UartComm()
{
	this->cleanup();
}

void UartComm::init(ActualCommandParams actualCommandParams)
{
	if (isOpen())
	{
		close();
	}
	const char * c = actualCommandParams.commPort.c_str();
	port.open(c);

	const boost::asio::serial_port_base::flow_control FLOW(boost::asio::serial_port_base::flow_control::none);
	const boost::asio::serial_port_base::stop_bits STOP(boost::asio::serial_port_base::stop_bits::one);

	if (actualCommandParams.enParity)
	{
		const boost::asio::serial_port_base::parity PARITY(boost::asio::serial_port_base::parity::even);
		port.set_option(PARITY);
	}
	else
	{
		const boost::asio::serial_port_base::parity PARITY(boost::asio::serial_port_base::parity::none);
		port.set_option(PARITY);
	}
	

	port.set_option(boost::asio::serial_port_base::character_size(8));
	port.set_option(FLOW);
	port.set_option(STOP);

	uint32_t initBaudRate;
	if ((actualCommandParams.protocol == Protocol::UART) ||
		(actualCommandParams.protocol == Protocol::USB) ||
		(actualCommandParams.protocol == Protocol::I2C)
		)
	{
		initBaudRate = actualCommandParams.baudRate + 1;
	}

#if defined(_WIN32) || defined(_WIN64)

	port.set_option(boost::asio::serial_port::baud_rate(initBaudRate));

#elif defined(__APPLE__)

	int fd = port.native_handle();
	ioctl(fd, _IOW('T', 2, speed_t), &initBaudRate, 1);

#else

	struct termios2 tio;
	int fd = port.native_handle();
	ioctl(fd, TCGETS2, &tio);
	tio.c_cflag &= ~CBAUD;
	tio.c_cflag |= 0010000;
	tio.c_ispeed = initBaudRate;
	tio.c_ospeed = initBaudRate;
	ioctl(fd, TCSETS2, &tio);

#endif

	std::this_thread::sleep_for(std::chrono::milliseconds(1000));

#if defined(_WIN32) || defined(_WIN64)

	port.set_option(boost::asio::serial_port::baud_rate(actualCommandParams.baudRate));

#elif defined(__APPLE__)

	ioctl(fd, _IOW('T', 2, speed_t), &actualCommandParams.baudRate, 1);

#else

	ioctl(fd, TCGETS2, &tio);
	tio.c_cflag &= ~CBAUD;
	tio.c_cflag |= 0010000;
	tio.c_ispeed = actualCommandParams.baudRate;
	tio.c_ospeed = actualCommandParams.baudRate;
	ioctl(fd, TCSETS2, &tio);

#endif

	std::this_thread::sleep_for(std::chrono::milliseconds(1000));
	actualBaudRate = actualCommandParams.baudRate;

	if (actualCommandParams.family == Family::P4xx)
	{
		transmitBufferInit432();
	}
}
/*******************************************************************************
*Class:       UartComm
*Function:
*Description:
*Parameters:
*Returns:
*******************************************************************************/
void UartComm::transmitBufferInit432()
{
	const uint8_t loopInitialization = 10;
	bool initP432 = false;
	uint8_t uartTxBuffer[1] = { 0xFF };

	for (uint8_t i = 0; i < loopInitialization; i++)
	{
		boost::system::error_code ec;
		boost::asio::write(port, boost::asio::buffer(uartTxBuffer, 1), ec);
		if (ec)
		{
			throw std::runtime_error("Error from writing occurs...");
		}

		headerBuffer[0] = 0xFF;
		receiveBuffer(-1);
		if (headerBuffer[0] == 0x00)
		{
			initP432 = true;
			break;
		}
	}

	if (initP432 == false)
	{
		throw std::runtime_error("Initialization of BSL P432 failed... Exit the scripter...");
	}
	else
	{
		std::cout << "Initialization of BSL P432 succeed..." << std::endl;
	}
}

/*******************************************************************************
*Class:       UartComm
*Function:
*Description:
*Parameters:
*Returns:
*******************************************************************************/
void UartComm::transmitBuffer(DataBuffer dataBuffer)
{
	uint8_t uartTxBuffer[UART_TX_BUFFER_SIZE];

	cleanRxBuffer();
	cleanHeaderBuffer();

	uartTxBuffer[0] = 0x80; //Header byte
	uartTxBuffer[1] = (uint8_t)(dataBuffer.size & 0xFF);
	uartTxBuffer[2] = (uint8_t)((dataBuffer.size >> 8) & 0xFF);

	crc = 0xFFFF;
	for (uint16_t i = 0; i < dataBuffer.size; i++)
	{
		uartTxBuffer[i + 3] = dataBuffer.data[i];
		checkSumCalculate(dataBuffer.data[i]);
	}

	uartTxBuffer[dataBuffer.size + 3] = getCheckSumLow(); //CKL
	uartTxBuffer[dataBuffer.size + 4] = getCheckSumHigh(); //CKH

	uint16_t numBytesSend = dataBuffer.size + 5;

	boost::system::error_code ec;
	boost::asio::write(port, boost::asio::buffer(uartTxBuffer, numBytesSend), ec);
	if (ec)
	{
		throw std::runtime_error("Error from writing occurs...");
	}

	bytesWrite += (dataBuffer.size - 4); // this belongs to RX_DATA_BLOCK and similar functions
	                                     // it counts the pure data without command byte and address bytes
}

/*******************************************************************************
*Class:       UartComm
*Function:
*Description:
*Parameters:
*Returns:
*******************************************************************************/
uint8_t* UartComm::receiveBuffer(int32_t size)
{
	uint32_t msDelay = 5000;
	if (!isOpen())
	{
		return 0;
	}
	//Reset all index in buffer
	resetIdxRead();
 	//start the first reading for ack
	setTimer(msDelay);
	startRead(headerBuffer, expSize);

	boost::system::error_code ec;
 	while (io_service.run_one(ec))
	{
		if (readEvent)
		{
			if (bytesReceived > 0)
			{
				actSize += bytesReceived;
				if (actSize == expSize)
				{
					bool protocolFine = fsmReceiveBuffer(size);
					// all data received, state machine goes back to waitAck
					if ((waitAck) || !protocolFine)
					{
						timer.cancel(ec);
						break;
					}
				}
			}

			if (waitAck)
			{
				startRead(headerBuffer, expSize - actSize);
			}
			else if (waitHeader)
			{
				startRead(&headerBuffer[1 + actSize], expSize - actSize);
			}
			else
			{
				startRead(&rxBuffer[cnt + actSize], expSize - actSize);
			}
		}
		else if (timerEvent)
		{
			cancelled = false;
			port.cancel(ec);
			break;
		}

		if (io_service.stopped())
		{
			io_service.reset();
		}
	}
	//Let cancelled tasks finish
	io_service.run(ec);
	io_service.reset();
	return &rxBuffer[0];
}

void UartComm::close()
{
	port.close();
}

void UartComm::changeBaudRate(uint32_t baudRate)
{
	port.set_option(boost::asio::serial_port::baud_rate(baudRate));
	actualBaudRate = baudRate;
	std::this_thread::sleep_for(std::chrono::milliseconds(1000));
}


uint32_t UartComm::retBytesWritten()
{
	return bytesWrite;
}

uint32_t UartComm::retBytesRead()
{
	return pureData;
}

uint32_t UartComm::retBytesPacketRead()
{
	return cnt;
}

uint8_t* UartComm::retHeaderBuffer()
{
	return &headerBuffer[0];
}

void UartComm::cleanRxBuffer()
{
	for (uint16_t i = 0; i < 100; i++)
	{
		rxBuffer[i] = 0;
	}
}

void UartComm::cleanHeaderBuffer()
{
	headerBuffer[0] = 0xFF;
	for (uint16_t i = 1; i < 4; i++)
	{
		headerBuffer[i] = 0;
	}
}

uint8_t UartComm::getCheckSumLow()
{
	return (uint8_t)(crc & 0x00FF);
}

uint8_t UartComm::getCheckSumHigh()
{
	return (uint8_t)((crc >> 8) & 0x00FF);
}

void UartComm::sendByte(uint8_t *data)
{
	boost::system::error_code ec;
	boost::asio::write(port, boost::asio::buffer(data, 1), ec);
	std::this_thread::sleep_for(std::chrono::microseconds(100));
}

void UartComm::checkSumCalculate(uint8_t data)
{
	uint8_t x;
	x = ((crc >> 8) ^ data) & 0xff;
	x ^= x >> 4;
	crc = (crc << 8) ^ (x << 12) ^ (x << 5) ^ x;
}

inline bool UartComm::fsmReceiveBuffer(int32_t size)
{
	bool retVal = true;

	if (size > 0)
	{
		if (waitAck)
		{
			//special case for command that gives back only one byte without 
			//wrapping bytes
			if (size == -1)
			{
				waitAck = true;
				waitHeader = false;
				waitData = false;
				actSize = 0;
				//renew expSize for reading the header bytes
				expSize = 1;
			}
			//these conditions are for the normal wrapping bsl packet
			else if (headerBuffer[0] == 0x00)
			{
				waitAck = false;
				waitHeader = true;
				waitData = false;
				actSize = 0;
				//renew expSize for reading the header bytes
				expSize = 3;
			}
			else
			{
				retVal = false;
			}

		}
		else if (waitHeader)
		{
			if (headerBuffer[1] == 0x80)
			{
				waitAck = false;
				waitHeader = false;
				waitData = true;
				actSize = 0;
				//renew expSize for reading the  data bytes
				expSize = (uint32_t)(((((uint16_t)headerBuffer[3]) << 8) | headerBuffer[2]) + 2);
			}
			else
			{
				retVal = false;
			}
		}
		else if (waitData)
		{

			waitAck = false;
			waitHeader = false;
			waitData = true;

			//crc calculation
			uint16_t crcBsl = (((uint16_t)rxBuffer[(cnt + bytesReceived) - 1]) << 8) | rxBuffer[(cnt + bytesReceived) - 2];
			crc = 0xFFFF;

			for (uint32_t i = cnt; i < (cnt + (expSize - 2)); i++)
			{
				checkSumCalculate(rxBuffer[i]);
			}

			if (crcBsl != crc)
			{
				retVal = false;
			}

			//update idx
			cnt += (expSize - 2);
			pureData += (expSize - 3);

			if (pureData == size)
			{
				waitAck = true;
				waitHeader = false;
				waitData = false;
				actSize = 0;
				//renew expSize for reading again the ack
				expSize = 1;
			}
			else
			{
				waitAck = false;
				waitHeader = true;
				waitData = false;
				actSize = 0;
				//renew expSize for reading again the hader bytes
				expSize = 3;
			}
		}
	}
	return retVal;
}

void UartComm::resetIdxRead()
{
	actSize = 0;
	cnt = 0;
	bytesReceived = 0;
	pureData = 0;
	expSize = 1;
	waitAck = true;
	waitHeader = false;
	waitData = false;
}

void UartComm::setTimer(uint32_t duration)
{
	timerEvent = false;

	if (&timer)
	{
		boost::system::error_code ec;
		timer.expires_from_now(boost::posix_time::milliseconds(duration), ec);
		timer.async_wait(bind(&UartComm::onTimer, this, boost::asio::placeholders::error));
	}
}

void UartComm::onTimer(const boost::system::error_code& ec)
{
	timerEvent = (ec != boost::asio::error::operation_aborted);
}

void UartComm::startRead(uint8_t* buf, size_t numBytes)
{
	bytesReceived = 0;
	readEvent = false;
	async_read(port, boost::asio::buffer(buf, numBytes), bind(&UartComm::onRead, this, boost::asio::placeholders::error,
		boost::asio::placeholders::bytes_transferred));
}

void UartComm::onRead(const boost::system::error_code& ec, size_t numBytes)
{
	readEvent = (ec != boost::asio::error::operation_aborted);
	bytesReceived = numBytes;
}

void UartComm::cleanup()
{
	if (isOpen())
	{
		boost::system::error_code ec = port.close(ec);
	}
}

bool UartComm::isOpen() const
{
	return &port && port.is_open();
}
  
void UartComm::cancel()
{
	cancelled = true;
	boost::system::error_code ec;
	if (&timer && timer.expires_from_now(boost::posix_time::milliseconds(0), ec) > 0)
	{
		timer.async_wait(bind(&UartComm::onTimer, this, boost::asio::placeholders::error));
	}
}
  







