/*
BSL-Scripter-3.1.0.0

==UsbComm.cpp==

Copyright (c) 2008  2015 Texas Instruments Incorporated

All rights reserved not granted herein.

Limited License.

Texas Instruments Incorporated grants a world-wide, royalty-free,
non-exclusive license under copyrights and patents it now or
hereafter owns or controls to make, have made, use, import,
offer to sell and sell ("Utilize") this software subject to
the terms herein.  With respect to the foregoing patent license,
such license is granted  solely to the extent that any such patent
is necessary to Utilize the software alone.  The patent license
shall not apply to any combinations which include this software,
other than combinations with devices manufactured by or for TI
(TI Devices). No hardware patent is licensed hereunder.

Redistributions must preserve existing copyright notices and
reproduce this license (including the above copyright notice
and the disclaimer and (if applicable) source code license
limitations below) in the documentation and/or other materials
provided with the distribution

Redistribution and use in binary form, without modification,
are permitted provided that the following conditions are met:

*	No reverse engineering, decompilation, or disassembly of
this software is permitted with respect to any software
provided in binary form.

*	any redistribution and use are licensed by TI for use only
with TI Devices.

*	Nothing shall obligate TI to provide you with source code
for the software licensed and provided to you in object code.

If software source code is provided to you, modification and
redistribution of the source code are permitted provided that
the following conditions are met:

*	any redistribution and use of the source code, including any
resulting derivative works, are licensed by TI for use only
with TI Devices.

*	any redistribution and use of any object code compiled from
the source code and any resulting derivative works, are licensed
by TI for use only with TI Devices.

Neither the name of Texas Instruments Incorporated nor the names
of its suppliers may be used to endorse or promote products derived
from this software without specific prior written permission.

DISCLAIMER.

THIS SOFTWARE IS PROVIDED BY TI AND TIS LICENSORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TIS LICENSORS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Version:	Release:		Author:
3.0.1.0		08/07/2015		Fatmawati Santosa
3.1.0.0		10/26/2015		Fatmawati Santosa

*/
#include "stdafx.h"

UsbComm::UsbComm()
	: handle(nullptr)
{	

}

UsbComm::~UsbComm()
{	

}

void UsbComm::init(ActualCommandParams actualCommandParams)
{
	if (hid_init())
	{
		throw std::runtime_error("HID init failed");
	}

	UsbComm::handle = hid_open(VID, PID, nullptr);

	if (!handle)
	{
		throw std::runtime_error("Unable to open the HID device");
	}

}

void UsbComm::transmitBufferInit432()
{
	throw std::runtime_error("No USB communication from P432...");
}

void UsbComm::transmitBuffer(DataBuffer dataBuffer)
{
	cleanRxBuffer();
	cleanHeaderBuffer();

	uint8_t usbSendBuffer[USB_TX_BUFFER_SIZE];

	usbSendBuffer[0] = 0x3F;
	usbSendBuffer[1] = dataBuffer.size;

	for (uint32_t i = 0; i < dataBuffer.size; i++)
	{
		usbSendBuffer[i + 2] = dataBuffer.data[i];
	}

	int32_t res = hid_write(handle, usbSendBuffer, (dataBuffer.size + 2));

	if (res < 0)
	{
		throw std::runtime_error("Data is not completely transmitted...");
	}

	bytesWrite += (dataBuffer.size - 4);
}

uint8_t* UsbComm::receiveBuffer(int32_t size)
{
	double numOfTimes = (double)(size) / 61.0;
	uint32_t numOfTimesInt = (uint32_t)ceil(numOfTimes);
	uint32_t numBytesRx = size + (3 * numOfTimesInt); //total bytes received
	uint32_t receiveBytes = 0; 
	uint32_t idx = 0; 

	if (size > 0)
	{
		if (numOfTimesInt == 1) //one packet
		{
			int32_t res = hid_read_timeout(handle, &rxBuffer[idx], numBytesRx, MS_DELAY_RX);
			if (res < 0)
			{
				std::cout << "Data is not completely received..." << std::endl;
			}
		}
		else //multiple packets
		{
			for (uint32_t i = 0; i < numOfTimesInt; i++)
			{
				if (i == (numOfTimesInt - 1)) //last packet
				{
					receiveBytes = numBytesRx - ((61 + 3) * (numOfTimesInt - 1));
				}
				else
				{
					receiveBytes = (61 + 3);
				}
				int32_t res = hid_read_timeout(handle, &rxBuffer[idx], receiveBytes, MS_DELAY_RX);
				if (res < 0)
				{
					std::cout << "Data is not completely received..." << std::endl;
					break;
				}
				idx += receiveBytes;
			}
		}
	}

	uint32_t k = 0; //new big buffer index
	uint32_t i = 0;	//count per 64
	
	headerBuffer[0] = rxBuffer[0];
	headerBuffer[1] = rxBuffer[1];

	for (uint32_t j = 0; j < numBytesRx; j++)
	{
		if (i > 1)
		{
			rxBuffer[k] = rxBuffer[j];
			k++;
		}
		i++;
		if (i == 64)
		{
			i = 0;
		}
	}
	
	bytesRead += k;
	cnt = size +  numOfTimesInt;

	return &rxBuffer[0];
}

void UsbComm::close()
{
	hid_close(handle);
}

void UsbComm::changeBaudRate(uint32_t baudRate)
{
	throw std::runtime_error("Change Baud Rate is not applicable for USB communication...");
}

uint32_t UsbComm::retBytesWritten()
{
	return bytesWrite;
}

uint32_t UsbComm::retBytesRead()
{
	return bytesRead;
}

uint8_t*   UsbComm::retHeaderBuffer()
{
	return &headerBuffer[0];
}

void UsbComm::cleanHeaderBuffer()
{
	headerBuffer[0] = 0;
	headerBuffer[1] = 0;
}

void UsbComm::cleanRxBuffer()
{
	for (uint32_t i = 0; i < 100; i++)
	{
		rxBuffer[i] = 0;
	}
}

uint32_t UsbComm::retBytesPacketRead()
{
	return cnt;
}